__title__ = 'eciespy'
__version__ = '0.1.2'
__description__ = 'Elliptic Curve Integrated Encryption Scheme for secp256k1 in Python'
__url__ = 'https://github.com/kigawas/eciespy'
__author__ = 'Weiliang Li'
__author_email__ = 'to.be.impressive@gmail.com'
__license__ = 'Apache-2.0'



import os
import unittest

from ecies import encrypt, decrypt
from ecies.utils import sha256, generate_eth_key, aes_encrypt, aes_decrypt


class TestCrypt(unittest.TestCase):

    def setUp(self):
        self.test_string = b'this is a test'
        self.big_data = b'0' * 1024 * 1024 * 100  # 100 MB

    def test_hash(self):
        self.assertEqual(sha256(b'0' * 16).hex()[:8], 'fcdb4b42')

    def test_elliptic(self):
        data = self.test_string
        k = generate_eth_key()
        prvhex = k.to_hex()
        pubhex = k.public_key.to_hex()
        self.assertEqual(data, decrypt(prvhex, encrypt(pubhex, data)))

    def test_aes(self):
        data = self.big_data
        key = os.urandom(16)
        self.assertEqual(data, aes_decrypt(key, aes_encrypt(key, data)))



import argparse
import sys

from ecies import encrypt, decrypt
from ecies.utils import generate_eth_key
from ecies.__version__ import __description__


def readablize(b: bytes) -> str:
    try:
        return b.decode()
    except ValueError:
        return b.hex()


def main():
    parser = argparse.ArgumentParser(description=__description__)

    parser.add_argument('-e', '--encrypt', action='store_true', help='encrypt with public key, not compatible with -d')
    parser.add_argument('-d', '--decrypt', action='store_true', help='decrypt with private key, not compatible with -e')
    parser.add_argument('-g', '--generate', action='store_true', help='generate ethereum key pair')
    parser.add_argument(
        '-k', '--key', type=argparse.FileType('r'), help='public or private key file'
    )

    parser.add_argument(
        '-D', '--data', nargs='?', type=argparse.FileType('rb'), default=sys.stdin,
        help='file to encrypt or decrypt, if not specified, it will read from stdin'
    )

    parser.add_argument(
        '-O', '--out', nargs='?', type=argparse.FileType('wb'), default=sys.stdout,
        help='encrypted or decrypted file, if not specified, it will write to stdout'
    )

    args = parser.parse_args()
    if args.generate:
        k = generate_eth_key()
        prv, pub, addr = k.to_hex(), k.public_key.to_hex(), k.public_key.to_checksum_address()
        print('Private: {}\nPublic: {}\nAddress: {}'.format(prv, pub, addr))
        return

    if args.encrypt == args.decrypt:
        parser.print_help()
        return

    if not args.key:
        parser.print_help()
        return

    key = args.key.read().strip()
    if args.encrypt:
        plaintext = args.data.read()
        if isinstance(plaintext, str):
            plaintext = plaintext.encode()
        data = encrypt(key, plaintext)
        if args.out == sys.stdout:
            data = data.hex()
    else:
        ciphertext = args.data.read()
        if isinstance(ciphertext, str):
            # if not bytes, suppose hex string
            ciphertext = bytes.fromhex(ciphertext.strip())
        data = decrypt(key, ciphertext)
        if args.out == sys.stdout:
            # if binary data, print hex; if not, print utf8
            data = readablize(data)

    args.out.write(data)


if __name__ == '__main__':
    main()



import hashlib

from Cryptodome.Cipher import AES

from coincurve import PrivateKey, PublicKey
from coincurve.utils import get_valid_secret

from eth_keys import keys
from eth_utils import decode_hex

AES_CIPHER_MODE = AES.MODE_GCM


def sha256(msg: bytes) -> bytes:
    '''
    Calculate sha256 hash.

    Parameters
    ----------
    msg: bytes
        message to hash

    Returns
    -------
    bytes
        sha256 hash in bytes

    >>> sha256(b'0'*16).hex()[:8] == 'fcdb4b42'
    True
    '''
    h = hashlib.sha256()
    h.update(msg)
    return h.digest()


def generate_key() -> PrivateKey:
    '''
    Generate random (or disposable) EC private key

    Returns
    -------
    coincurve.PrivateKey
        A secp256k1 key

    >>> k = generate_key()
    '''
    return PrivateKey(get_valid_secret())


def generate_eth_key() -> keys.PrivateKey:
    '''
    Generate random eth private key

    Returns
    -------
    eth_keys.keys.PrivateKey
        An ethereum key

    >>> k = generate_eth_key()
    '''
    return keys.PrivateKey(get_valid_secret())


def hex2pub(pub_hex: str) -> PublicKey:
    '''
    Convert ethereum hex to EllipticCurvePublicKey
    The hex should be 65 bytes, but ethereum public key only has 64 bytes
    So have to add \x04

    Parameters
    ----------
    pub_hex: str
        Ethereum public key hex string

    Returns
    -------
    coincurve.PublicKey
        A secp256k1 public key calculated from ethereum public key hex string

    >>> data = b'0'*32
    >>> data_hash = sha256(data)
    >>> eth_prv = generate_eth_key()
    >>> cc_prv = hex2prv(eth_prv.to_hex())
    >>> eth_prv.sign_msg_hash(data_hash).to_bytes() == cc_prv.sign_recoverable(data)
    True
    >>> pubhex = eth_prv.public_key.to_hex()
    >>> computed_pub = hex2pub(pubhex)
    >>> computed_pub == cc_prv.public_key
    True
    '''
    uncompressed = decode_hex(pub_hex)
    if len(uncompressed) == 64:
        uncompressed = b'\x04' + uncompressed

    return PublicKey(uncompressed)


def hex2prv(prv_hex: str) -> PrivateKey:
    '''
    Convert ethereum hex to EllipticCurvePrivateKey

    Parameters
    ----------
    prv_hex: str
        Ethereum private key hex string

    Returns
    -------
    coincurve.PrivateKey
        A secp256k1 private key calculated from ethereum private key hex string

    >>> k = generate_eth_key()
    >>> prvhex = k.to_hex()
    >>> pubhex = k.public_key.to_hex()
    >>> computed_prv = hex2prv(prvhex)
    >>> computed_prv.to_int() == int(k.to_hex(), 16)
    True
    '''
    return PrivateKey(decode_hex(prv_hex))


def derive(private_key: PrivateKey, peer_public_key: PublicKey) -> bytes:
    '''
    Key exchange between private key and peer's public key,
    `derive(k1, k2.public_key)` should be equal to `derive(k2, k1.public_key)`.

    Parameters
    ----------
    private_key: coincurve.PrivateKey
        A secp256k1 private key
    peer_public_key: coincurve.PublicKey
        Peer's public key

    Returns
    -------
    bytes
        A secret key used for symmetric encryption

    >>> from coincurve import PrivateKey
    >>> ke1 = generate_eth_key()
    >>> ke2 = generate_eth_key()
    >>> k1 = hex2prv(ke1.to_hex())
    >>> k2 = hex2prv(ke2.to_hex())
    >>> derive(k1, k2.public_key) == derive(k2, k1.public_key)
    True
    '''
    return private_key.ecdh(peer_public_key.format())


def aes_encrypt(key: bytes, plain_text: bytes) -> bytes:
    '''
    AES-GCM encryption

    Parameters
    ----------
    key: bytes
        AES session key, derived from two secp256k1 keys
    plain_text: bytes
        Plain text to encrypt

    Returns
    -------
    bytes
        nonce(16 bytes) + tag(16 bytes) + encrypted data
    '''
    aes_cipher = AES.new(key, AES_CIPHER_MODE)

    encrypted, tag = aes_cipher.encrypt_and_digest(plain_text)
    cipher_text = bytearray()
    cipher_text.extend(aes_cipher.nonce)
    cipher_text.extend(tag)
    cipher_text.extend(encrypted)
    return bytes(cipher_text)


def aes_decrypt(key: bytes, cipher_text: bytes) -> bytes:
    '''
    AES-GCM decryption

    Parameters
    ----------
    key: bytes
        AES session key, derived from two secp256k1 keys
    cipher_text: bytes
        Encrypted plain text:
            nonce(16 bytes) + tag(16 bytes) + encrypted data

    Returns
    -------
    bytes
        Plain text

    >>> data = b'this is test data'
    >>> key = get_valid_secret()
    >>> aes_decrypt(key, aes_encrypt(key, data)) == data
    True
    >>> import os
    >>> key = os.urandom(32)
    >>> aes_decrypt(key, aes_encrypt(key, data)) == data
    True
    '''
    nonce = cipher_text[:16]
    tag = cipher_text[16:32]
    ciphered_data = cipher_text[32:]

    aes_cipher = AES.new(key, AES_CIPHER_MODE, nonce=nonce)
    return aes_cipher.decrypt_and_verify(ciphered_data, tag)


if __name__ == '__main__':
    import doctest
    doctest.testmod()




from ecies.utils import generate_key, hex2prv, hex2pub, derive, aes_encrypt, aes_decrypt

__all__ = ['encrypt', 'decrypt']


def encrypt(receiver_pubhex: str, msg: bytes) -> bytes:
    '''
    Encrypt with eth public key

    Parameters
    ----------
    receiver_pubhex: str
        Receiver's ethereum public key hex string
    msg: bytes
        Data to encrypt

    Returns
    -------
    bytes
        Encrypted data
    '''
    disposable_key = generate_key()
    receiver_pubkey = hex2pub(receiver_pubhex)
    aes_key = derive(disposable_key, receiver_pubkey)
    cipher_text = aes_encrypt(aes_key, msg)
    return disposable_key.public_key.format(False) + cipher_text


def decrypt(receiver_prvhex: str, msg: bytes) -> bytes:
    '''
    Decrypt with eth private key

    Parameters
    ----------
    receiver_pubhex: str
        Receiver's ethereum private key hex string
    msg: bytes
        Data to decrypt

    Returns
    -------
    bytes
        Plain text
    '''
    pubkey = msg[0:65]  # pubkey's length is 65 bytes
    encrypted = msg[65:]
    sender_public_key = hex2pub(pubkey.hex())
    private_key = hex2prv(receiver_prvhex)
    aes_key = derive(private_key, sender_public_key)
    return aes_decrypt(aes_key, encrypted)



'''
 _______  _______ _________ _______  _______  _______
(  ____ \(  ____ \\__   __/(  ____ \(  ____ \(  ____ )|\     /|
| (    \/| (    \/   ) (   | (    \/| (    \/| (    )|( \   / )
| (__    | |         | |   | (__    | (_____ | (____)| \ (_) /
|  __)   | |         | |   |  __)   (_____  )|  _____)  \   /
| (      | |         | |   | (            ) || (         ) (
| (____/\| (____/\___) (___| (____/\/\____) || )         | |
(_______/(_______/\_______/(_______/\_______)|/          \_/

'''
import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
about = {}  # type: dict

with open(os.path.join(here, 'ecies', '__version__.py'), 'r') as f:
    exec(f.read(), about)

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name=about['__title__'],
    version=about['__version__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    url=about['__url__'],
    description=about['__description__'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    license=about['__license__'],
    packages=find_packages(),
    install_requires=[
        'eth-keys',
        'pysha3',
        'pycryptodomex',
        'coincurve',
    ],
    entry_points={
        'console_scripts': [
            'eciespy = ecies.__main__:main'
        ],
    },
    keywords=(
        'secp256k1',
        'crypto',
        'elliptic curves',
        'ecies',
        'bitcoin',
        'ethereum',
        'cryptocurrency'
    ),
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Security :: Cryptography',
    )
)
